import { N } from './prelude.js';
import * as Cmp from '@prelude/cmp';
export { Cmp };
export declare type CmpWithRight<K> = (b: K) => Cmp.R;
export declare type RbTree<T, K> = {
    cmp: Cmp.t<K>;
    key: (value: T) => K;
    root: N<T>;
};
export declare type t<T, K> = RbTree<T, K>;
/** Range query. */
export declare type Query<K> = undefined | ({} | {
    $l: K;
} | {
    $le: K;
} & ({} | {
    $r: K;
} | {
    $re: K;
}));
/** @returns Red-black tree based on element-to-key mapping function and comparision over keys function. */
export declare const of: <T, K>(cmp: Cmp.t<K>, key: (value: T) => K) => RbTree<T, K>;
export declare const get: <T, K>(tree: RbTree<T, K>, key: K) => [T | undefined, number];
export declare const getv: <T, K>(tree: RbTree<T, K>, key: K) => T | undefined;
export declare const getc: <T, K>(tree: RbTree<T, K>, key: K) => number;
export declare const has: <T, K>(tree: RbTree<T, K>, key: K) => boolean;
/**
 * Inserts provided `value`.
 * If element already exists, `merge` is used to produce final element (defaults to identity function).
 */
export declare const insert: <T, K>(tree: RbTree<T, K>, element: NonNullable<T>, n?: number, merge?: (a: T, b: T) => T) => void;
/**
 * Shifts left-most element (ie. minimum element).
 * @returns `undefined` or shifted element.
 */
export declare const maybeShiftCount: <T, K>(tree: RbTree<T, K>) => [T | undefined, number];
/**
 * @returns removed left-most element.
 * @throws if tree is empty.
 */
export declare const shiftCount: <T, K>(tree: RbTree<T, K>) => [value: T, count: number];
export declare const shift: <T, K>(tree: RbTree<T, K>) => T;
export declare const each: <T, K>(tree: RbTree<T, K>) => Generator<T, any, unknown>;
export declare const countLeft_: <T, K>(_: N<T>, key: (value: T) => K, cmp: CmpWithRight<K>, inclusive: boolean) => number;
export declare const count: <T, K>(tree: RbTree<T, K>, query?: Query<K>) => number;
declare const delete_: <T, K>(tree: RbTree<T, K>, key: K, i?: number) => [T | undefined, number];
export { delete_ as delete };
/** Assert local invariant - red nodes don't have red children. */
export declare const assertLocal: <T, K>(tree: RbTree<T, K>) => void;
/** Assert global invariant - paths from nil leafs up to the root go through same number of black nodes. */
export declare const assertGlobal: <T, K>(tree: RbTree<T, K>) => void;
export declare const assertMonotonic: <T, K>(tree: RbTree<T, K>) => void;
export declare const assert: <T, K>(tree: RbTree<T, K>) => void;
